@@ -24,7 +24,7 @@ import logging
 from PySide import QtCore
 
 from leap.common.check import leap_assert, leap_assert_type
-from leap.common.config import get_path_prefix
+from leap.bitmask.util import get_path_prefix
 
 logger = logging.getLogger(__name__)
 
@@ -65,22 +65,17 @@ class LeapSettings(object):
     PROPERPROVIDER_KEY = "ProperProvider"
     REMEMBER_KEY = "RememberUserAndPass"
     DEFAULTPROVIDER_KEY = "DefaultProvider"
+    AUTOSTARTEIP_KEY = "AutoStartEIP"
     ALERTMISSING_KEY = "AlertMissingScripts"
     GATEWAY_KEY = "Gateway"
+    PINNED_KEY = "Pinned"
 
     # values
     GATEWAY_AUTOMATIC = "Automatic"
 
-    def __init__(self, standalone=False):
-        """
-        Constructor
-
-        :param standalone: parameter used to define the location of
-        the config
-        :type standalone: bool
-        """
-        settings_path = os.path.join(
-            get_path_prefix(standalone=standalone), "leap", self.CONFIG_NAME)
+    def __init__(self):
+        settings_path = os.path.join(get_path_prefix(),
+                                     "leap", self.CONFIG_NAME)
 
         self._settings = QtCore.QSettings(settings_path,
                                           QtCore.QSettings.IniFormat)
@@ -132,8 +127,8 @@ class LeapSettings(object):
         # other things, not just the directories
         providers = []
         try:
-            providers_path = os.path.join(
-                get_path_prefix(), "leap", "providers")
+            providers_path = os.path.join(get_path_prefix(),
+                                          "leap", "providers")
             providers = os.listdir(providers_path)
         except Exception as e:
             logger.debug("Error listing providers, assume there are none. %r"
@@ -141,6 +136,22 @@ class LeapSettings(object):
 
         return providers
 
+    def is_pinned_provider(self, domain):
+        """
+        Returns True if the domain 'domain' is pinned with the application.
+                False otherwise.
+
+        :param provider: provider domain
+        :type provider: str
+
+        :rtype: bool
+        """
+        leap_assert(len(domain) > 0, "We need a nonempty domain.")
+        pinned_key = "{0}/{1}".format(domain, self.PINNED_KEY)
+        result = to_bool(self._settings.value(pinned_key, False))
+
+        return result
+
     def get_selected_gateway(self, provider):
         """
         Returns the configured gateway for the given provider.
@@ -292,6 +303,24 @@ class LeapSettings(object):
         else:
             self._settings.setValue(self.DEFAULTPROVIDER_KEY, provider)
 
+    def get_autostart_eip(self):
+        """
+        Gets whether the app should autostart EIP.
+
+        :rtype: bool
+        """
+        return to_bool(self._settings.value(self.AUTOSTARTEIP_KEY, False))
+
+    def set_autostart_eip(self, autostart):
+        """
+        Sets whether the app should autostart EIP.
+
+        :param autostart: True if we should try to autostart EIP.
+        :type autostart: bool
+        """
+        leap_assert_type(autostart, bool)
+        self._settings.setValue(self.AUTOSTARTEIP_KEY, autostart)
+
     def get_alert_missing_scripts(self):
         """
         Returns the setting for alerting of missing up/down scripts.
